@@ -58,7 +58,9 @@ import Data.Word (Word8, Word16, Word32)
 import Network.URI (URI(..), uriToString, nullURI, parseURI)
 import Numeric (readHex)
 import Data.Time.Locale.Compat (defaultTimeLocale)
-import Text.PrettyPrint.Free (Pretty(..), (<+>), char, hsep, punctuate, space, text)
+import Data.Text.Prettyprint.Doc (Pretty(..), (<+>), hsep, punctuate, space)
+import Codec.Encryption.OpenPGP.Types.Internal.PrettyUtils (prettyLBS)
+
 
 type Exportability = Bool
 type TrustLevel = Word8
@@ -141,21 +143,21 @@ instance FutureVal SymmetricAlgorithm where
 instance Hashable SymmetricAlgorithm
 
 instance Pretty SymmetricAlgorithm where
-    pretty Plaintext = text "plaintext"
-    pretty IDEA = text "IDEA"
-    pretty TripleDES = text "3DES"
-    pretty CAST5 = text "CAST-128"
-    pretty Blowfish = text "Blowfish"
-    pretty ReservedSAFER = text "(reserved) SAFER"
-    pretty ReservedDES = text "(reserved) DES"
-    pretty AES128 = text "AES-128"
-    pretty AES192 = text "AES-192"
-    pretty AES256 = text "AES-256"
-    pretty Twofish = text "Twofish"
-    pretty Camellia128 = text "Camellia-128"
-    pretty Camellia192 = text "Camellia-192"
-    pretty Camellia256 = text "Camellia-256"
-    pretty (OtherSA sa) = text "unknown symmetric algorithm" <+> pretty sa
+    pretty Plaintext = pretty "plaintext"
+    pretty IDEA = pretty "IDEA"
+    pretty TripleDES = pretty "3DES"
+    pretty CAST5 = pretty "CAST-128"
+    pretty Blowfish = pretty "Blowfish"
+    pretty ReservedSAFER = pretty "(reserved) SAFER"
+    pretty ReservedDES = pretty "(reserved) DES"
+    pretty AES128 = pretty "AES-128"
+    pretty AES192 = pretty "AES-192"
+    pretty AES256 = pretty "AES-256"
+    pretty Twofish = pretty "Twofish"
+    pretty Camellia128 = pretty "Camellia-128"
+    pretty Camellia192 = pretty "Camellia-192"
+    pretty Camellia256 = pretty "Camellia-256"
+    pretty (OtherSA sa) = pretty "unknown symmetric algorithm" <+> pretty sa
 
 $(ATH.deriveJSON ATH.defaultOptions ''SymmetricAlgorithm)
 
@@ -179,8 +181,8 @@ instance FutureFlag NotationFlag where
 instance Hashable NotationFlag
 
 instance Pretty NotationFlag where
-    pretty HumanReadable = text "human-readable"
-    pretty (OtherNF o) = text "unknown notation flag type" <+> pretty o
+    pretty HumanReadable = pretty "human-readable"
+    pretty (OtherNF o) = pretty "unknown notation flag type" <+> pretty o
 
 $(ATH.deriveJSON ATH.defaultOptions ''NotationFlag)
 
@@ -192,7 +194,7 @@ instance Newtype ThirtyTwoBitTimeStamp Word32 where
     unpack (ThirtyTwoBitTimeStamp o) = o
 
 instance Pretty ThirtyTwoBitTimeStamp where
-    pretty = text . formatTime defaultTimeLocale "%Y%m%d-%H%M%S" . posixSecondsToUTCTime . realToFrac
+    pretty = pretty . formatTime defaultTimeLocale "%Y%m%d-%H%M%S" . posixSecondsToUTCTime . realToFrac
 
 $(ATH.deriveJSON ATH.defaultOptions ''ThirtyTwoBitTimeStamp)
 
@@ -212,7 +214,7 @@ instance Newtype ThirtyTwoBitDuration Word32 where
     unpack (ThirtyTwoBitDuration o) = o
 
 instance Pretty ThirtyTwoBitDuration where
-    pretty = text . concat . unfoldr durU . unpack
+    pretty = pretty . concat . unfoldr durU . unpack
 
 $(ATH.deriveJSON ATH.defaultOptions ''ThirtyTwoBitDuration)
 
@@ -236,8 +238,8 @@ instance FutureFlag RevocationClass where
 instance Hashable RevocationClass
 
 instance Pretty RevocationClass where
-    pretty SensitiveRK = text "sensitive"
-    pretty (RClOther o) = text "unknown revocation class" <+> pretty o
+    pretty SensitiveRK = pretty "sensitive"
+    pretty (RClOther o) = pretty "unknown revocation class" <+> pretty o
 
 $(ATH.deriveJSON ATH.defaultOptions ''RevocationClass)
 
@@ -284,16 +286,16 @@ instance FutureVal PubKeyAlgorithm where
 instance Hashable PubKeyAlgorithm
 
 instance Pretty PubKeyAlgorithm where
-    pretty RSA = text "RSA"
-    pretty DeprecatedRSAEncryptOnly = text "(deprecated) RSA encrypt-only"
-    pretty DeprecatedRSASignOnly = text "(deprecated) RSA sign-only"
-    pretty ElgamalEncryptOnly = text "Elgamal encrypt-only"
-    pretty DSA = text "DSA"
-    pretty ECDH = text "ECDH"
-    pretty ECDSA = text "ECDSA"
-    pretty ForbiddenElgamal = text "(forbidden) Elgamal"
-    pretty DH = text "DH"
-    pretty (OtherPKA pka) = text "unknown pubkey algorithm type" <+> pretty pka
+    pretty RSA = pretty "RSA"
+    pretty DeprecatedRSAEncryptOnly = pretty "(deprecated) RSA encrypt-only"
+    pretty DeprecatedRSASignOnly = pretty "(deprecated) RSA sign-only"
+    pretty ElgamalEncryptOnly = pretty "Elgamal encrypt-only"
+    pretty DSA = pretty "DSA"
+    pretty ECDH = pretty "ECDH"
+    pretty ECDSA = pretty "ECDSA"
+    pretty ForbiddenElgamal = pretty "(forbidden) Elgamal"
+    pretty DH = pretty "DH"
+    pretty (OtherPKA pka) = pretty "unknown pubkey algorithm type" <+> pretty pka
 
 $(ATH.deriveJSON ATH.defaultOptions ''PubKeyAlgorithm)
 
@@ -327,7 +329,7 @@ instance Newtype SpacedFingerprint TwentyOctetFingerprint where
     unpack (SpacedFingerprint o) = o
 
 instance Pretty SpacedFingerprint where
-    pretty = hsep . punctuate space . map hsep . chunksOf 5 . map text . chunksOf 4 . take 40 . bsToHexUpper . unTOF . unpack
+    pretty = hsep . punctuate space . map hsep . chunksOf 5 . map pretty . chunksOf 4 . take 40 . bsToHexUpper . unTOF . unpack
 
 bsToHexUpper :: ByteString -> String
 bsToHexUpper = map toUpper . BLC8.unpack . B16L.encode
@@ -360,7 +362,10 @@ instance A.FromJSON EightOctetKeyId where
     parseJSON _            = mzero
 
 newtype NotationName = NotationName {unNotationName :: ByteString}
-    deriving (Data, Eq, Generic, Hashable, Ord, Pretty, Show, Typeable)
+    deriving (Data, Eq, Generic, Hashable, Ord, Show, Typeable)
+
+instance Pretty NotationName where
+    pretty = prettyLBS . unNotationName
 
 instance Newtype NotationName ByteString where
     pack = NotationName
@@ -374,7 +379,10 @@ instance A.FromJSON NotationName where
     parseJSON _            = mzero
 
 newtype NotationValue = NotationValue {unNotationValue :: ByteString}
-    deriving (Data, Eq, Generic, Hashable, Ord, Pretty, Show, Typeable)
+    deriving (Data, Eq, Generic, Hashable, Ord, Show, Typeable)
+
+instance Pretty NotationValue where
+    pretty = prettyLBS . unNotationValue
 
 instance Newtype NotationValue ByteString where
     pack = NotationValue
@@ -424,14 +432,14 @@ instance FutureVal HashAlgorithm where
 instance Hashable HashAlgorithm
 
 instance Pretty HashAlgorithm where
-    pretty DeprecatedMD5 = text "(deprecated) MD5"
-    pretty SHA1 = text "SHA-1"
-    pretty RIPEMD160 = text "RIPEMD-160"
-    pretty SHA256 = text "SHA-256"
-    pretty SHA384 = text "SHA-384"
-    pretty SHA512 = text "SHA-512"
-    pretty SHA224 = text "SHA-224"
-    pretty (OtherHA ha) = text "unknown hash algorithm type" <+> pretty ha
+    pretty DeprecatedMD5 = pretty "(deprecated) MD5"
+    pretty SHA1 = pretty "SHA-1"
+    pretty RIPEMD160 = pretty "RIPEMD-160"
+    pretty SHA256 = pretty "SHA-256"
+    pretty SHA384 = pretty "SHA-384"
+    pretty SHA512 = pretty "SHA-512"
+    pretty SHA224 = pretty "SHA-224"
+    pretty (OtherHA ha) = pretty "unknown hash algorithm type" <+> pretty ha
 
 $(ATH.deriveJSON ATH.defaultOptions ''HashAlgorithm)
 
@@ -463,11 +471,11 @@ instance FutureVal CompressionAlgorithm where
 instance Hashable CompressionAlgorithm
 
 instance Pretty CompressionAlgorithm where
-    pretty Uncompressed = text "uncompressed"
-    pretty ZIP = text "ZIP"
-    pretty ZLIB = text "zlib"
-    pretty BZip2 = text "bzip2"
-    pretty (OtherCA ca) = text "unknown compression algorithm type" <+> pretty ca
+    pretty Uncompressed = pretty "uncompressed"
+    pretty ZIP = pretty "ZIP"
+    pretty ZLIB = pretty "zlib"
+    pretty BZip2 = pretty "bzip2"
+    pretty (OtherCA ca) = pretty "unknown compression algorithm type" <+> pretty ca
 
 $(ATH.deriveJSON ATH.defaultOptions ''CompressionAlgorithm)
 
@@ -491,8 +499,8 @@ instance FutureFlag KSPFlag where
 instance Hashable KSPFlag
 
 instance Pretty KSPFlag where
-    pretty NoModify = text "no-modify"
-    pretty (KSPOther o) = text "unknown keyserver preference flag type" <+> pretty o
+    pretty NoModify = pretty "no-modify"
+    pretty (KSPOther o) = pretty "unknown keyserver preference flag type" <+> pretty o
 
 $(ATH.deriveJSON ATH.defaultOptions ''KSPFlag)
 
@@ -534,14 +542,14 @@ instance FutureFlag KeyFlag where
 instance Hashable KeyFlag
 
 instance Pretty KeyFlag where
-    pretty GroupKey = text "group"
-    pretty AuthKey = text "auth"
-    pretty SplitKey = text "split"
-    pretty EncryptStorageKey = text "encrypt-storage"
-    pretty EncryptCommunicationsKey = text "encrypt-communications"
-    pretty SignDataKey = text "sign-data"
-    pretty CertifyKeysKey = text "certify-keys"
-    pretty (KFOther o) = text "unknown key flag type" <+> pretty o
+    pretty GroupKey = pretty "group"
+    pretty AuthKey = pretty "auth"
+    pretty SplitKey = pretty "split"
+    pretty EncryptStorageKey = pretty "encrypt-storage"
+    pretty EncryptCommunicationsKey = pretty "encrypt-communications"
+    pretty SignDataKey = pretty "sign-data"
+    pretty CertifyKeysKey = pretty "certify-keys"
+    pretty (KFOther o) = pretty "unknown key flag type" <+> pretty o
 
 $(ATH.deriveJSON ATH.defaultOptions ''KeyFlag)
 
@@ -576,12 +584,12 @@ instance FutureVal RevocationCode where
 instance Hashable RevocationCode
 
 instance Pretty RevocationCode where
-    pretty NoReason = text "no reason"
-    pretty KeySuperseded = text "key superseded"
-    pretty KeyMaterialCompromised = text "key material compromised"
-    pretty KeyRetiredAndNoLongerUsed = text "key retired and no longer used"
-    pretty UserIdInfoNoLongerValid = text "user-ID info no longer valid"
-    pretty (RCoOther o) = text "unknown revocation code" <+> pretty o
+    pretty NoReason = pretty "no reason"
+    pretty KeySuperseded = pretty "key superseded"
+    pretty KeyMaterialCompromised = pretty "key material compromised"
+    pretty KeyRetiredAndNoLongerUsed = pretty "key retired and no longer used"
+    pretty UserIdInfoNoLongerValid = pretty "user-ID info no longer valid"
+    pretty (RCoOther o) = pretty "unknown revocation code" <+> pretty o
 
 $(ATH.deriveJSON ATH.defaultOptions ''RevocationCode)
 
@@ -607,8 +615,8 @@ instance Hashable a => Hashable (Set a) where
     hashWithSalt salt = hashWithSalt salt . Set.toList
 
 instance Pretty FeatureFlag where
-    pretty ModificationDetection = text "modification-detection"
-    pretty (FeatureOther o) = text "unknown feature flag type" <+> pretty o
+    pretty ModificationDetection = pretty "modification-detection"
+    pretty (FeatureOther o) = pretty "unknown feature flag type" <+> pretty o
 
 $(ATH.deriveJSON ATH.defaultOptions ''FeatureFlag)
 
@@ -694,22 +702,22 @@ instance FutureVal SigType where
 instance Hashable SigType
 
 instance Pretty SigType where
-    pretty BinarySig = text "binary"
-    pretty CanonicalTextSig = text "canonical-text"
-    pretty StandaloneSig = text "standalone"
-    pretty GenericCert = text "generic"
-    pretty PersonaCert = text "persona"
-    pretty CasualCert = text "casual"
-    pretty PositiveCert = text "positive"
-    pretty SubkeyBindingSig = text "subkey-binding"
-    pretty PrimaryKeyBindingSig = text "primary-key-binding"
-    pretty SignatureDirectlyOnAKey = text "signature directly on a key"
-    pretty KeyRevocationSig = text "key-revocation"
-    pretty SubkeyRevocationSig = text "subkey-revocation"
-    pretty CertRevocationSig = text "cert-revocation"
-    pretty TimestampSig = text "timestamp"
-    pretty ThirdPartyConfirmationSig = text "third-party-confirmation"
-    pretty (OtherSig o) = text "unknown signature type" <+> pretty o
+    pretty BinarySig = pretty "binary"
+    pretty CanonicalTextSig = pretty "canonical-pretty"
+    pretty StandaloneSig = pretty "standalone"
+    pretty GenericCert = pretty "generic"
+    pretty PersonaCert = pretty "persona"
+    pretty CasualCert = pretty "casual"
+    pretty PositiveCert = pretty "positive"
+    pretty SubkeyBindingSig = pretty "subkey-binding"
+    pretty PrimaryKeyBindingSig = pretty "primary-key-binding"
+    pretty SignatureDirectlyOnAKey = pretty "signature directly on a key"
+    pretty KeyRevocationSig = pretty "key-revocation"
+    pretty SubkeyRevocationSig = pretty "subkey-revocation"
+    pretty CertRevocationSig = pretty "cert-revocation"
+    pretty TimestampSig = pretty "timestamp"
+    pretty ThirdPartyConfirmationSig = pretty "third-party-confirmation"
+    pretty (OtherSig o) = pretty "unknown signature type" <+> pretty o
 
 $(ATH.deriveJSON ATH.defaultOptions ''SigType)
 
@@ -735,9 +743,9 @@ data SignaturePayload = SigV3 SigType ThirtyTwoBitTimeStamp EightOctetKeyId PubK
 instance Hashable SignaturePayload
 
 instance Pretty SignaturePayload where
-    pretty (SigV3 st ts eoki pka ha w16 mpis) = text "signature v3" <> char ':' <+> pretty st <+> pretty ts <+> pretty eoki <+> pretty pka <+> pretty ha <+> pretty w16 <+> (pretty . NE.toList) mpis
-    pretty (SigV4 st pka ha hsps usps w16 mpis) = text "signature v4" <> char ':' <+> pretty st <+> pretty pka <+> pretty ha <+> pretty hsps <+> pretty usps <+> pretty w16 <+> (pretty . NE.toList) mpis
-    pretty (SigVOther t bs) = text "unknown signature v" <> pretty t <> char ':' <+> pretty (BL.unpack bs)
+    pretty (SigV3 st ts eoki pka ha w16 mpis) = pretty "signature v3" <> pretty ':' <+> pretty st <+> pretty ts <+> pretty eoki <+> pretty pka <+> pretty ha <+> pretty w16 <+> (pretty . NE.toList) mpis
+    pretty (SigV4 st pka ha hsps usps w16 mpis) = pretty "signature v4" <> pretty ':' <+> pretty st <+> pretty pka <+> pretty ha <+> pretty hsps <+> pretty usps <+> pretty w16 <+> (pretty . NE.toList) mpis
+    pretty (SigVOther t bs) = pretty "unknown signature v" <> pretty t <> pretty ':' <+> pretty (BL.unpack bs)
 
 instance A.ToJSON SignaturePayload where
     toJSON (SigV3 st ts eoki pka ha w16 mpis) = A.toJSON (st, ts, eoki, pka, ha, w16, NE.toList mpis)
@@ -774,31 +782,31 @@ data SigSubPacketPayload = SigCreationTime ThirtyTwoBitTimeStamp
 instance Hashable SigSubPacketPayload
 
 instance Pretty SigSubPacketPayload where
-    pretty (SigCreationTime ts) = text "creation-time" <+> pretty ts
-    pretty (SigExpirationTime d) = text "sig expiration time" <+> pretty d
-    pretty (ExportableCertification e) = text "exportable certification" <+> pretty e
-    pretty (TrustSignature tl ta) = text "trust signature" <+> pretty tl <+> pretty ta
-    pretty (RegularExpression apdre) = text "regular expression" <+> pretty apdre
-    pretty (Revocable r) = text "revocable" <+> pretty r
-    pretty (KeyExpirationTime d) = text "key expiration time" <+> pretty d
-    pretty (PreferredSymmetricAlgorithms sas) = text "preferred symmetric algorithms" <+> pretty sas
-    pretty (RevocationKey rcs pka tof) = text "revocation key" <+> pretty (Set.toList rcs) <+> pretty pka <+> pretty tof
-    pretty (Issuer eoki) = text "issuer" <+> pretty eoki
-    pretty (NotationData nfs nn nv) = text "notation data" <+> pretty (Set.toList nfs) <+> pretty nn <+> pretty nv
-    pretty (PreferredHashAlgorithms phas) = text "preferred hash algorithms" <+> pretty phas
-    pretty (PreferredCompressionAlgorithms pcas) = text "preferred compression algorithms" <+> pretty pcas
-    pretty (KeyServerPreferences kspfs) = text "keyserver preferences" <+> pretty (Set.toList kspfs)
-    pretty (PreferredKeyServer ks) = text "preferred keyserver" <+> pretty ks
-    pretty (PrimaryUserId p) = (if p then mempty else text "NOT ") <> text "primary user-ID"
-    pretty (PolicyURL u) = text "policy URL" <+> pretty u
-    pretty (KeyFlags kfs) = text "key flags" <+> pretty (Set.toList kfs)
-    pretty (SignersUserId u) = text "signer's user-ID" <+> pretty u
-    pretty (ReasonForRevocation rc rr) = text "reason for revocation" <+> pretty rc <+> pretty rr
-    pretty (Features ffs) = text "features" <+> pretty (Set.toList ffs)
-    pretty (SignatureTarget pka ha sh) = text "signature target" <+> pretty pka <+> pretty ha <+> pretty sh
-    pretty (EmbeddedSignature sp) = text "embedded signature" <+> pretty sp
-    pretty (UserDefinedSigSub t bs) = text "user-defined signature subpacket type" <+> pretty t <+> pretty (BL.unpack bs)
-    pretty (OtherSigSub t bs) = text "unknown signature subpacket type" <+> pretty t <+> pretty bs
+    pretty (SigCreationTime ts) = pretty "creation-time" <+> pretty ts
+    pretty (SigExpirationTime d) = pretty "sig expiration time" <+> pretty d
+    pretty (ExportableCertification e) = pretty "exportable certification" <+> pretty e
+    pretty (TrustSignature tl ta) = pretty "trust signature" <+> pretty tl <+> pretty ta
+    pretty (RegularExpression apdre) = pretty "regular expression" <+> prettyLBS apdre
+    pretty (Revocable r) = pretty "revocable" <+> pretty r
+    pretty (KeyExpirationTime d) = pretty "key expiration time" <+> pretty d
+    pretty (PreferredSymmetricAlgorithms sas) = pretty "preferred symmetric algorithms" <+> pretty sas
+    pretty (RevocationKey rcs pka tof) = pretty "revocation key" <+> pretty (Set.toList rcs) <+> pretty pka <+> pretty tof
+    pretty (Issuer eoki) = pretty "issuer" <+> pretty eoki
+    pretty (NotationData nfs nn nv) = pretty "notation data" <+> pretty (Set.toList nfs) <+> pretty nn <+> pretty nv
+    pretty (PreferredHashAlgorithms phas) = pretty "preferred hash algorithms" <+> pretty phas
+    pretty (PreferredCompressionAlgorithms pcas) = pretty "preferred compression algorithms" <+> pretty pcas
+    pretty (KeyServerPreferences kspfs) = pretty "keyserver preferences" <+> pretty (Set.toList kspfs)
+    pretty (PreferredKeyServer ks) = pretty "preferred keyserver" <+> prettyLBS ks
+    pretty (PrimaryUserId p) = (if p then mempty else pretty "NOT ") <> pretty "primary user-ID"
+    pretty (PolicyURL u) = pretty "policy URL" <+> pretty u
+    pretty (KeyFlags kfs) = pretty "key flags" <+> pretty (Set.toList kfs)
+    pretty (SignersUserId u) = pretty "signer's user-ID" <+> pretty u
+    pretty (ReasonForRevocation rc rr) = pretty "reason for revocation" <+> pretty rc <+> pretty rr
+    pretty (Features ffs) = pretty "features" <+> pretty (Set.toList ffs)
+    pretty (SignatureTarget pka ha sh) = pretty "signature target" <+> pretty pka <+> pretty ha <+> prettyLBS sh
+    pretty (EmbeddedSignature sp) = pretty "embedded signature" <+> pretty sp
+    pretty (UserDefinedSigSub t bs) = pretty "user-defined signature subpacket type" <+> pretty t <+> pretty (BL.unpack bs)
+    pretty (OtherSigSub t bs) = pretty "unknown signature subpacket type" <+> pretty t <+> prettyLBS bs
 
 instance A.ToJSON SigSubPacketPayload where
     toJSON (SigCreationTime ts) = object [T.pack "sigCreationTime" .= ts]
@@ -850,7 +858,7 @@ data SigSubPacket = SigSubPacket {
   } deriving (Data, Eq, Generic, Show, Typeable)
 
 instance Pretty SigSubPacket where
-    pretty x = (if _sspCriticality x then char '*' else mempty) <> (pretty . _sspPayload) x
+    pretty x = (if _sspCriticality x then pretty '*' else mempty) <> (pretty . _sspPayload) x
 
 instance Hashable SigSubPacket
 
@@ -863,8 +871,8 @@ data KeyVersion = DeprecatedV3 | V4
 instance Hashable KeyVersion
 
 instance Pretty KeyVersion where
-    pretty DeprecatedV3 = text "(deprecated) v3"
-    pretty V4 = text "v4"
+    pretty DeprecatedV3 = pretty "(deprecated) v3"
+    pretty V4 = pretty "v4"
 
 $(ATH.deriveJSON ATH.defaultOptions ''KeyVersion)
 
@@ -907,10 +915,10 @@ instance FutureVal DataType where
     toFVal o = OtherData o
 
 instance Pretty DataType where
-    pretty BinaryData = text "binary"
-    pretty TextData = text "text"
-    pretty UTF8Data = text "UTF-8"
-    pretty (OtherData o) = text "other data type " <+> pretty o
+    pretty BinaryData = pretty "binary"
+    pretty TextData = pretty "text"
+    pretty UTF8Data = pretty "UTF-8"
+    pretty (OtherData o) = pretty "other data type " <+> pretty o
 
 $(ATH.deriveJSON ATH.defaultOptions ''DataType)
 
@@ -948,10 +956,10 @@ data S2K = Simple HashAlgorithm
 instance Hashable S2K
 
 instance Pretty S2K where
-    pretty (Simple ha) = text "simple S2K," <+> pretty ha
-    pretty (Salted ha salt) = text "salted S2K," <+> pretty ha <+> pretty salt
-    pretty (IteratedSalted ha salt icount) = text "iterated-salted S2K," <+> pretty ha <+> pretty salt <+> pretty icount
-    pretty (OtherS2K t bs) = text "unknown S2K type" <+> pretty t <+> pretty (bsToHexUpper bs)
+    pretty (Simple ha) = pretty "simple S2K," <+> pretty ha
+    pretty (Salted ha salt) = pretty "salted S2K," <+> pretty ha <+> pretty salt
+    pretty (IteratedSalted ha salt icount) = pretty "iterated-salted S2K," <+> pretty ha <+> pretty salt <+> pretty icount
+    pretty (OtherS2K t bs) = pretty "unknown S2K type" <+> pretty t <+> pretty (bsToHexUpper bs)
 
 instance A.ToJSON S2K where
     toJSON (Simple ha) = A.toJSON ha
@@ -979,8 +987,8 @@ instance FutureVal ImageFormat where
 instance Hashable ImageFormat
 
 instance Pretty ImageFormat where
-    pretty JPEG = text "JPEG"
-    pretty (OtherImage o) = text "unknown image format" <+> pretty o
+    pretty JPEG = pretty "JPEG"
+    pretty (OtherImage o) = pretty "unknown image format" <+> pretty o
 
 $(ATH.deriveJSON ATH.defaultOptions ''ImageFormat)
 
@@ -993,7 +1001,7 @@ instance Ord ImageHeader where
 instance Hashable ImageHeader
 
 instance Pretty ImageHeader where
-    pretty (ImageHV1 f) = text "imghdr v1" <+> pretty f
+    pretty (ImageHV1 f) = pretty "imghdr v1" <+> pretty f
 
 $(ATH.deriveJSON ATH.defaultOptions ''ImageHeader)
 
@@ -1010,8 +1018,8 @@ instance Ord UserAttrSubPacket where
     compare (OtherUASub t1 b1) (OtherUASub t2 b2) = compare t1 t2 <> compare b1 b2
 
 instance Pretty UserAttrSubPacket where
-    pretty (ImageAttribute ih d) = text "image-attribute" <+> pretty ih <+> pretty (BL.unpack d)
-    pretty (OtherUASub t bs) = text "unknown attribute type" <> pretty t <+> pretty (BL.unpack bs)
+    pretty (ImageAttribute ih d) = pretty "image-attribute" <+> pretty ih <+> pretty (BL.unpack d)
+    pretty (OtherUASub t bs) = pretty "unknown attribute type" <> pretty t <+> pretty (BL.unpack bs)
 
 instance A.ToJSON UserAttrSubPacket where
     toJSON (ImageAttribute ih d) = A.toJSON (ih, BL.unpack d)
