@@ -20,7 +20,7 @@ import qualified Data.Aeson as A
 import Data.Data (Data)
 import Data.Hashable (Hashable(..))
 import Data.Typeable (Typeable)
-import Text.PrettyPrint.Free (Pretty(..), (<+>), tupled)
+import Data.Text.Prettyprint.Doc (Pretty(..), (<+>), tupled)
 
 newtype DSA_PublicKey = DSA_PublicKey {unDSA_PublicKey :: DSA.PublicKey}
     deriving (Data, Eq, Generic, Show, Typeable)
