@@ -25,7 +25,7 @@ import Data.List.NonEmpty (NonEmpty)
 import Data.Text (Text)
 import Data.Typeable (Typeable)
 import Data.Word (Word8)
-import Text.PrettyPrint.Free (Pretty(..))
+import Data.Text.Prettyprint.Doc (Pretty(..))
 
 class Packet a where
     data PacketType a :: *
