@@ -39,7 +39,7 @@ import Data.Binary.Get (runGetOrFail, Get)
 import Data.Binary.Put (runPut)
 import Data.Text (Text)
 import Data.Time.Clock.POSIX (posixSecondsToUTCTime)
-import Text.PrettyPrint.Free (pretty)
+import Data.Text.Prettyprint.Doc (pretty)
 
 import qualified Data.Conduit as DC
 import qualified Data.Conduit.Binary as CB
